@@ -35,7 +35,9 @@
 # include <windows.h>
 #endif
 
-#include <iconv.h>
+#ifdef HAVE_ICONV
+# include <iconv.h>
+#endif
 
 #include "gettext.h"
 #define _(string) gettext(string)
@@ -54,7 +56,7 @@ static int vasprintf(char **strp, const char *format, va_list args)
      * and its return value is standards compliant. This is true for the MinGW version
      * of vsnprintf(), but not for Microsofts version (Visual Studio etc.)!
      */
-    int length = vsnprintf(nullptr, 0, format, args);
+    int length = vsnprintf(NULL, 0, format, args);
     if (length > std::numeric_limits<int>::max() - 1
             || !(*strp = static_cast<char *>(malloc(length + 1))))
     {
@@ -388,11 +390,27 @@ namespace str
         }
     }
 
+    std::string human_readable_time(int64_t microseconds)
+    {
+        int64_t hours = microseconds / (1000000ll * 60ll * 60ll);
+        microseconds -= hours * (1000000ll * 60ll * 60ll);
+        int64_t minutes = microseconds / (1000000ll * 60ll);
+        microseconds -= minutes * (1000000ll * 60ll);
+        int64_t seconds = microseconds / 1000000ll;
+        std::string hr;
+        if (hours > 0)
+        {
+            hr = str::from(hours) + (minutes < 10 ? ":0" : ":");
+        }
+        hr += str::from(minutes) + (seconds < 10 ? ":0" : ":") + str::from(seconds);
+        return hr;
+    }
+
     /* Get the name of the user's character set */
     std::string localcharset()
     {
 #ifdef HAVE_NL_LANGINFO
-        std::string bak = setlocale(LC_CTYPE, nullptr);
+        std::string bak = setlocale(LC_CTYPE, NULL);
         setlocale(LC_CTYPE, "");
         char *charset = nl_langinfo(CODESET);
         setlocale(LC_CTYPE, bak.c_str());
@@ -421,6 +439,7 @@ namespace str
             return src;
         }
 
+#ifdef HAVE_ICONV
         iconv_t cd = iconv_open(to_charset.c_str(), from_charset.c_str());
         if (cd == reinterpret_cast<iconv_t>(static_cast<size_t>(-1)))
         {
@@ -446,7 +465,7 @@ namespace str
         }
         char *outbuf = orig_outbuf;
 
-        size_t s = iconv(cd, const_cast<char **>(&inbuf), &inbytesleft, &outbuf, &outbytesleft);
+        size_t s = iconv(cd, const_cast<ICONV_CONST char **>(&inbuf), &inbytesleft, &outbuf, &outbytesleft);
         int saved_errno = errno;
         iconv_close(cd);
         if (s == static_cast<size_t>(-1))
@@ -467,5 +486,8 @@ namespace str
         }
         free(orig_outbuf);
         return dst;
+#else
+        throw exc(str::asprintf(_("Cannot convert %s to %s."), from_charset.c_str(), to_charset.c_str()), ENOSYS);
+#endif
     }
 }
