@@ -28,7 +28,7 @@
 #include <string>
 #include <cstdarg>
 #include <cerrno>
-#include <cstdint>
+#include <stdint.h>
 
 #include "exc.h"
 
@@ -93,6 +93,7 @@ namespace str
     /* Convert various values to human readable strings */
     std::string human_readable_memsize(const uintmax_t size);
     std::string human_readable_length(const double length);
+    std::string human_readable_time(int64_t microseconds);
 
     /* Get the name of the user's character set */
     std::string localcharset();
