@@ -115,7 +115,7 @@ extern "C" int gtatool_dimension_extract(int argc, char *argv[])
             std::vector<uintmax_t> indices(hdri.dimensions());
             for (uintmax_t i = 0; i < hdri.elements(); i++)
             {
-                void *e = element_loop.read();
+                const void *e = element_loop.read();
                 hdri.linear_index_to_indices(i, &(indices[0]));
                 if (indices[dim] == ind)
                 {
